@echo off

rem   ----------------------------------------------------------------------------
rem   -----     Rename to "slideShowParams.bat" to use this file <================
rem   -----
rem   -----     Parameters for makeSlideShow.bat (in the same folder)
rem   ----------------------------------------------------------------------------
rem   -----
rem   ----- More or less important parameters are defined in this file
rem   ----- Important parameters are marked with some !!!!!!!!!!
rem   -----
rem   ----- Before usage, edit the images with a program like Adobe Lightroom.
rem   ----- This makes possible to use the image metadata for title and caption.
rem   ----- So you avoid extensive editing the slideshow file after generating.
rem   -----
rem   ----- peter@danninger.eu                                      12.05.2020
rem   ----------------------------------------------------------------------------

rem   UTF-8 Codepage (important evtl. for TITEL)
rem   Achtung: Datei als UTF8 (ohne BOM) speichern !!!!!
chcp 65001

rem   Just a heading for this file, not used further:
rem             ==========================================
rem   !!!!!!!!!!              Example file                =========
rem             ==========================================

rem   nur für Logging in makeSlideShow.bat
rem
set PARAMSDATE=12.05.2020
set PARAMSHEADER=English Example file (HdS Faschingsgaudi 2016)

rem   name of the thumbnailshow file, default: index.html
set FILE1=index.html

rem   name of the  slideshow file, default: slideshow.html:
set FILE2=slideshow.html

rem   name of the folder for the slides, default: slides
set SLIDES=slides

rem   name of the folder for the thumbnails, default: thumbnails
set THUMBNAILS=thumbnails

rem             =========================================================
rem   !!!!!!!!!!     Name of the folder for the original(big) images     =========
rem             =========================================================
rem   evtl. bigger images for downloading or printing
rem   default is oriPics or false (if not defined)
rem   if false, the images in slides are used for downloading or printing
rem
set ORIPICDIR=oriPics

rem             ===============================================
rem   !!!!!!!!!!        Enlarge slides on big displays         =========
rem             ===============================================
rem   true or false, default: true
rem
set ENLARGE=true

rem             ====================================================
rem   !!!!!!!!!!     Page title for thumbnailshow and slideshow     =========
rem             ====================================================
rem   for the HTML header and the Thumbnailshow headline
rem   evtl. also for unique image title,
rem   if MAKE_PAGETITLE=true, look below
rem
set TITEL=HdS Faschingsgaudi 2016

rem             ======================================================
rem   !!!!!!!!!!  Image name without the 3 digit sequence at the end  ========
rem             ======================================================
rem   Convention: 3 digit sequence at the end !!!
rem   same for all images: slides, thumbnails and oriPics
rem   p.e.: Mekong_2011_001 .... Mekong_2011_511
rem
set BILDNAME=FaschingHdS2016_

rem             ====================================
rem   !!!!!!!!!!    Number of images, max. 999     =====================
rem             ====================================
rem   The maximum of 999 is because of the 3 digit sequence,
rem   but this should be enough for common slideshows.
rem
set BILDANZ=10
rem
rem   If BILDANZ > SCROLLANZ more thumbnails will be loaded depending
rem   of the position of the scrollbar.
rem   Default: 64, evtl. false, if functionality not wanted.
rem
set SCROLLANZ=64

rem             ================================================
rem   !!!!!!!!!!      Path to the script and style folders      =============
rem             ================================================
rem   These are the folders: js, css, fonts, help, icons
rem   and the file makeSlideShow.bat
rem   If there is only 1 slideshow, the scripts are evtl. locally: ./
rem   If there are more slideshows, the scripts are better in a shared folder: ../slideShow/
rem
set SCRIPTSDIR=./
rem
rem   If SCRIPTSDIR=./ you have evtl. a scripts and styles root folder
rem   for central maintenance.
rem   Example: ..\..\slideShow\ or false (default)
rem   Attention: Backslash !!!
rem
set SCRIPTSROOT=false

rem             =============================================
rem   ==========  generate thumbnailshow for www.senotto.de  ================
rem             =============================================
rem   This is a special case only for integrating in www.senotto.de.
rem   If the thumbnailshow is generated for www.senotto.de,
rem   the Breadcrumb-Navigation will be inserted. [true | false]
rem
set SENOTTO=false
rem
rem   only if "SENOTTO=true":
rem   path to the senotto root folder, p.e.: ../../, or false
rem
set SENOTTO_ROOT=false
rem
rem   only if "SENOTTO=true":
rem   name of the senotto home page, p.e.: Bilder und Videos, or false
rem
set SENOTTO_PARENT=false

rem             ===========================================
rem   ==========    Heading text over the thumbnailshow    =============
rem             ===========================================
rem   Example: click image to start slideshow, default: false
rem   Att.:    & \ < > ^ |    have to be escaped, p.e.:   ^^^<
rem   Note: extensive textes insert manually in the generated thumnailshow.
rem
set THUMB_TEXT=false

rem             =====================================
rem   ==========    Thumbnail height (50 ... 200)    ============================
rem             =====================================
rem   Default: 100
rem
set THUMB_HEIGHT=100

rem             ===========================================
rem   ==========    Common image title in the slideshow    ============
rem             ===========================================
rem   does make sense only if no individual image title. [true | false]
rem   look below ....
rem   if true, %TITEL% will be used (look above)
rem
set MAKE_PAGETITLE=false

rem             ==========================================================================
rem   ==========  Generate empty <div class="figTitle"> or <figcaption> in the slideshow  ==================
rem             ==========================================================================
rem   does make sense only if these will be edited manually in the slideshow. [true | false]
rem   To avoid this, use the image metadata, look below ....
rem
set EMPTYTITLE=false
set EMPTYCAPTION=false

rem             ======================================
rem   ==========     Common Slideshow Parameters      ====================
rem             ======================================

rem   Symbol for the Download-Button [true | false]:
set DOWNLOADBUT=true

rem   Symbol for the Print-Button [true | false]:
set PRINTBUT=true

rem   !!!!!!!!!! Evaluate the image metadata and show the Meta-Button [true | false]:
set METABUT=true

rem   !!!!!!!!!! Symbol for the GoogleMaps-Button [true | false]:
rem              only if METABUT=true
rem   To use this functionality for own sites you need a GoogleStaticMaps Key:
rem   To generate a personal key look at:
rem     "https://developers.google.com/maps/documentation/static-maps/?hl=de"
rem
set GOOGLEMAPSBUT=true
rem   !!!!!!!!!! GoogleMaps Map [roadmap | satellite | hybrid | terrain]
rem   Default: hybrid
set GOOGLEMAPSMAP=hybrid
rem   Peter's GoogleMaps Key
rem   !!!!!!!!!! your personal GoogleMaps-Key as textstring in ""
set GOOGLEMAPSKEY="AIzaSyB1NQXQl6j3bKG9mkNQsp1yLmIYT-5JMcA"

rem   Symbol for the Exit-Button [true | false]:
set EXITBUT=true

rem   Symbol for the FullScreen-Button [true | false]:
set FULLSCREENBUT=true

rem   Symbol for the Help-Button [true | false]:
set HELPBUT=true

rem   Symbol for the Play-Pause Button [true | false]:
set PLAYPAUSEBUT=true

rem   !!!!!!!!!! Playmode 0: forward, 1: backward, 2: random, 3: ask me
set PLAYMODUS=0

rem   !!!!!!!!!! anable wrap-around on Show-Start and Show-End [true | false]:
set WRAPAROUND=false

rem   !!!!!!!!!! Display time if automatic playmode, min 2 sec:
set AUTOSPEED=3

rem   !!!!!!!!!! Transition time, max 3 sec, z.B: 1 sec:
set TRANSITION=1

rem   !!!!!!!!!! Waiting time for manual proceeding, 0.2 - 2 sec, z.B: 1 sec:
set WAITPROCEED=0.5

rem             ========================================================
rem   !!!!!!!!!!     evaluate image metadata for title and caption      =======
rem             ========================================================

rem   !!!!!!!!!! Use the image metadata to generate the image title [true | false]:
rem
set ADDTITLE=true
rem
rem   !!!!!!!!!! Use the image metadata to generate the image caption [true | false]:
rem
set ADDCAPTION=true
rem
rem   !!!!!!!!!! Define the metadata to be displayed, look below:
rem
set TITEL_LI_1=title
set CAPTION_LI_1=desc
set CAPTION_RE_1=date

rem   The title will be assembled as follows:
rem       TITEL_LI_1, TITEL_LI_2          TITEL_RE_1, TITEL_RE_2
rem
rem   The caption will be assembled as follows:
rem       CAPTION_LI_1, CAPTION_LI_2          CAPTION_RE_1, CAPTION_RE_2
rem
rem   ==> Following assignements are possible:
rem       title, headline, desc, rights, date, time, make, model, artist, userComment, focalLen,
rem       fNumber, subjDist, expTime, expBias, ISOSpeed, pixelX, pixelY, latGrad, lonGrad, ele,
rem       country, province, city, rating, label, exifVers, creaTool, name
rem   ==> special case name: filename of the image, evtl. alt-attribute
rem
rem   The definitions should be self-explaining.
rem   But it depends from the image editing program, which ones are defined.
rem   I've tested with Adobe Lightroom.
rem   Just try and error ......
rem   Attention: no plausibility test on generating

rem   Example:
rem
rem   TITEL_LI_1=title
rem   CAPTION_LI_1=desc
rem   CAPTION_RE_1=date

rem   Description of supported metadata:
rem   name:         Bildname                      (special case)
rem   title:        Titel                         (XMP | IPTC)
rem   headline:     Bildüberschrift               (XMP | IPTC)
rem   desc:         Beschreibung, Caption         (XMP | IPTC | EXIF)
rem   byline:       Künstler                      (IPTC)
rem   creator:      Ersteller                     (XMP)
rem   rights:       Copyright                     (XMP | IPTC | EXIF)
rem   date:         Aufnahmedatum                 (EXIF)
rem   make:         Kamera Hersteller             (EXIF)
rem   model:        Kamera Modell                 (EXIF)
rem   userComment:  Benutzer-Kommentar            (EXIF)
rem   artist:       Fotograf                      (EXIF)
rem   focalLen:     Objektiv Brennweite           (EXIF)
rem   fNumber:      Objektiv Blende               (EXIF)
rem   subjDist:     Objektiv Entfernung           (EXIF)
rem   expTime:      Kamera Belichtungszeit        (EXIF)
rem   expBias:      Kamera Belichtungskorrektur   (EXIF)
rem   ISOSpeed:     Kamera ISO Empfindlichkeit    (EXIF)
rem   pixelX:       Kamera Bildbreite             (EXIF)
rem   pixelY:       Kamera Bildhöhe               (EXIF)
rem   latGrad:      GPS Latitude (Breite)         (EXIF)
rem   lonGrad:      GPS Longitude (Höhe)          (EXIF)
rem   ele:          GPS Höhe                      (EXIF)
rem   country:      Land                          (XMP | IPTC)
rem   province:     Provinz                       (XMP | IPTC)
rem   city:         Stadt                         (XMP | IPTC)
rem   rating:       Bewertung, z.B: "4"           (XMP)
rem   label:        Markierung, z.B: "Grün"       (XMP)
rem   exifVers:     EXIF Version                  (EXIF)
rem   creaTool:     Foto-Software, z.B: Lighroom  (XMP)

rem   the end
